# Elcom Printer Board Communication driver

<!-- Table of contents -->
<details>
  <summary markdown="span">
    <b>Contents</b>
  </summary>

  [[_TOC_]]
</details>

**Date:** 2024-08-28

## General notes <a name="chapter__en_general_notes"></a>

The information in this document is provided as is and no guarantee or warranty
is given that the information is fit for any particular purpose. You may not use,
download, upload, copy, print, display, perform, reproduce, publish, license, post,
transmit, or distribute any information from this document in whole or in part
without the express written authorization of ELCOM, spoločnosť s ručením obmedzeným,
Prešov (“COMPANY”). COMPANY reserves the right to change this document at any time,
at its sole discretion, without previous notice.\
COMPANY reserves the right to seek all remedies available by law and in equity
for any violation of these Terms. Any rights not expressly granted herein
are reserved. COMPANY reserves the right to release information to the proper authorities,
as a result of a violation of these Terms, or unlawful acts, if the information
is subpoenaed and/or if we deem it necessary and/or appropriate. COMPANY hereby
disclaims all warranties and conditions with regard to this document, including
all implied warranties and conditions of fitness for a particular purpose, title,
and non-infringement. The information published in this document could include
inaccuracies, errors, or omissions, that be technical or not. Changes are periodically
added to the document. COMPANY may, but is not obligated to, make improvements
and/or changes in the content described herein at any time. Further to what precedes,
COMPANY, its managers and its employees shall in no event be held liable for any
special, direct, indirect, or consequential damages or any damages whatsoever resulting
from technical problems incurred by an equipment such as a breakdown, loss of use, data,
or profits, whether in an action of equity, contract, negligence, or other tortuous
action, arising out of or in connection with the use of this document.
Likewise, COMPANY shall in no event be held liable in case of force majeure.

## About the Document <a name="chapter__en_about_doc"></a>

### History <a name="chapter__en_history"></a>

| Date       | Description                                                                    |
|------------|--------------------------------------------------------------------------------|
| 2022-07-18 | Initial version                                                                |
| 2024-08-28 | Changed link to NFU application protocol to file within these developer guides |
|            |                                                                                |

## Introduction <a name="chapter__en_introduction"></a>

- The Elcom Printer Board Communication driver, `elcom_pbc` in short, provides support
  for communication with `Elcom Printer Board`, also called the non-fiscal unit (`NFU`),
  which is part of Elcom electronic cash registers based on the `Android` or `Linux`
  operating system.
- The `Elcom Printer Board` is connected to the main SoM module (running `Android` or `Linux`)
  via the `UART` serial interface and its main goal is to provide the printing functionality.
- The goal of the `elcom_pbc` driver is to cover the communication at the link layer
  and to provide a simple interface for reliable communication at higher communication
  layer.
- The driver is designed as a `Linux` kernel module, which is automatically loaded into
  the system at start-up.

## Communication with the Elcom Printer Board via device /dev/elcom_pbc <a name="chapter__en_communication_via_elcom_pbc_dev"></a>

- The driver interface is implemented by a device called `elcom_pbc`, which is available
  in the `/dev/` directory:
  ```
  /dev/elcom_pbc
  ```
- Device `/dev/elcom_pbc` is virtual file which serves for communication between application
  running in user space and `Elcom Printer Board`
- Before starting communication, it is necessary to open the `/dev/elcom_pbc` file with `rw`
  (read/write) rights. Writing into file is used to send a message to the `Elcom Printer Board`
  and reading from file is used to read a response. After the required communication
  is completed, it is necessary to close the file so that it is also available for other
  applications because multiple opening of the file is not allowed.
- The communication messages (commands) are written into `/dev/elcom_pbc` file in text form
  and are detaily described in communication protocol of the application layer.
- The communication protocol of the application layer for `Elcom Printer Board` is described in file:
    - `additional/NFU_x_xxx_xx/NFU_AppProtocol.pdf`
- Encoding page of texts sent to the `/dev/elcom_pbc` file must be according information obtained
  from command `GETDEVINFO`
- E.g. the command to obtain information about the `Elcom Printer Board` is written
  to the `/dev/elcom_pbc` file as text in the form
  ```
  GETDEVINFO\tREQ\n
  ```
  > :memo: **Note:**
  >   -  The characters \t and \n are represented by the hexadecimal code 0x09 and 0x0A
  >      (see the communication protocol of the application layer for more information)

- The response from the `Elcom Printer Board` can then be obtained by reading
  from the `/dev/elcom_pbc` file. After reading the entire response, the content
  of the file is cleared, which means that the next `read` will return zero bytes.
  The `read` function returns 0 in native code (C/C++), it returns -1 in Java,
  indicating the end of file. The response can be read only once after writing
  into the file.
- If an error occurs while communicating with the `Elcom Printer Board`, reading from the file
  will return an error or cause an IOException in Java that needs to be caught.

### Non-blocking reading from file <a name="chapter__en_non_blocking_reading_from_file"></a>

- Reading from the `/dev/elcom_pbc` file is non-blocking, meaning that the response
  may not be read when the first time the `read` function is called. This is mainly
  the case when reading from the file is called immediately after sending the message
  to the `Elcom Printer Board`. It is because the communication itself and the execution
  of the command in the `Elcom Printer Board` takes some time and the call to the `read`
  function is not blocked for the entire duration of the communication.
- In this case, the `read` function does not return the number of bytes read,
  but either the -EAGAIN error in native code (C/C++) or 0 read bytes in Java;
  see the Java code example. The caller must repeatedly call the `read` function
  until it returns a positive number of bytes read, or another error (IOException in Java).
  After an error, it is necessary to resend the message. If the `Elcom Printer Board`
  does not respond, the `read` function returns the -ETIMEDOUT error/exception.
    > :memo: **Note:**
    >   -  It is not necessary to insert any `sleep` function call between each repeated call
    >      of the `read` function because the driver itself already ensures that the given
    >      thread does not overload the CPU

## Sending communication messages from shell <a name="chapter__en_sending_messages_from_shell"></a>

- It is also possible to use the shell command line for easy communication with the `Elcom Printer Board`.
  It is available via ADB (Android Debug Bridge) in Android by entering the following command:
	```
  adb shell
  ```
  > :memo: **Note:**
  >   -  How to install and use ADB is not covered in this manual.

- To send a "Message" to the `Elcom Printer Board`, use the `echo` command (note that quotation
  marks are important):
  ```
	echo -e "Message" > /dev/elcom_pbc
  ```
  for example:
  ```
	echo -e "GETDEVINFO\tREQ\n" > /dev/elcom_pbc
  ```
- To read the response from the `Elcom Printer Board`, use the `cat` command:
  ```
  cat /dev/elcom_pbc
  ```
## Example in Java <a name="chapter__en_example_in_java"></a>

- The following example illustrates simple communication with the `Elcom Printer Board` in Java
  language. `RandomAccessFile` and `FileChannel` classes are used to work with the file.
    > :warning: **!!! WARNING !!!**
    >   -  It is not recommended to use `FileInputStream` and `FileOutputStream` because
    >      device `/dev/elcom_pbc` does not support multiple file openings, so objects of these
    >      classes can not work simultaneously.

- The function `makePrintBufferTest` in following example sends commands to the `Elcom Printer Board`
  to print set of 2 lines and then waits for printout finish. This example does not analyse
  the received messages in most cases, as it is not the subject of this manual.
    > :memo: **Note:**
    >   -  The following code is just a simple example, does not take into account all error states
    >      and only serves as inspiration on how to use /dev/elcom_pbc device in Java.
- Example:
  ```java
  import java.io.FileNotFoundException;
  import java.io.IOException;
  import java.io.RandomAccessFile;
  import java.nio.ByteBuffer;
  import java.nio.channels.FileChannel;

  public class PBCTest {
      public static void makePrintBufferTest(){
          String messageAddLine1 = "PRN_ABT\tREQ\tThis is line 1.\n";
          String messageAddLine2 = "PRN_ABT\tREQ\tThis is line 2.\n";
          String messagePrintBuf = "PRN_PBT\tREQ\n";
          String messageObtainProgress = "oP\tREQ\n";
          String response;

          try {
              // creating RandomAccessFile for writing
              RandomAccessFile elcom_pbc = new RandomAccessFile("/dev/elcom_pbc", "rw");

              // getting FileChannel from file
              FileChannel channel = elcom_pbc.getChannel();

              // add line 1 to buffer
              response = sendPBCMessage( channel, messageAddLine1 );
              // add line 2 to buffer
              response = sendPBCMessage( channel, messageAddLine2 );
              // print buffer
              response = sendPBCMessage( channel, messagePrintBuf );

              // wait for print buffer finish
              do {
                  // obtain progress
                  response = sendPBCMessage( channel, messageObtainProgress );
              }while ( !checkObtainProgressResponseFinished( response ) );

              channel.close();    // closing file channel
              elcom_pbc.close();  // closing RandomAccess file
          } catch (FileNotFoundException e) {
              e.printStackTrace();
          } catch (IOException e) {
              e.printStackTrace();
          }
      };

      public static boolean checkObtainProgressResponseFinished(String response) {
          boolean finished = false;

          String[] respList = response.split( "[\t\n]");
          if( respList.length == 4 ) {
              int exceptCode = Integer.parseInt( respList[2] );
              int progress = Integer.parseInt( respList[3] );
              if( respList[0].equals("oP") && respList[1].equals("RSP") ) {
                  if( exceptCode == 0 ){
                      if( progress >= 100)
                          finished = true;    //finished with success
                  } else {
                      finished = true;        //finished with error
                  }
              } else {
                  finished = true;            //finished with error
              }
          } else {
              finished = true;                //finished with error
          }
          return finished;
      }

      public static String sendPBCMessage(FileChannel channel, String message) {
          String response = "";

          try {
              // creating and initializing ByteBuffer for reading/writing data
              ByteBuffer buffer = ByteBuffer.allocate(2560);

              buffer.clear();
              System.out.println("Send message: " + message);
              buffer.put(message.getBytes());
              // flip the buffer for writing into file
              buffer.flip();
              int numOfBytesWritten = channel.write(buffer); // writing into file
              System.out.println("Number of bytes written: " + numOfBytesWritten);

              // reading data from file channel into buffer
              buffer.clear();
              int numOfBytesRead;
              // if bytes read count is 0, repeat reading
              while ((numOfBytesRead = channel.read(buffer)) == 0);
              System.out.println("Number of bytes read: " + numOfBytesRead);
              if (numOfBytesRead > 0) {
                  // we need to flip the byte buffer before reading
                  buffer.flip();
                  byte[] rawBytes = new byte[numOfBytesRead];
                  buffer.get(rawBytes);
                  response = new String(rawBytes);
                  System.out.println("Received response: " + response);
              }
          } catch (IOException e) {
              e.printStackTrace();
          }
          return response;
      }
  }
  ```
